import hashlib
import binascii
import sys
import math

#       rc4.py - RC4, ARC4, ARCFOUR algorithm with random salt
#
#       Copyright (c) 2009 joonis new media
#       Author: Thimo Kraemer <thimo.kraemer@joonis.de>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
def crypt(data, key):
    """RC4 algorithm"""
    x = 0
    box = range(256)
    for i in range(256):
        x = (x + box[i] + ord(key[i % len(key)])) % 256
        box[i], box[x] = box[x], box[i]
    x = y = 0
    out = []
    for char in data:
        x = (x + 1) % 256
        y = (y + box[x]) % 256
        box[x], box[y] = box[y], box[x]
        out.append(chr(ord(char) ^ box[(box[x] + box[y]) % 256]))

    return ''.join(out)

#------------------------------------
#
#------------------------------------
if len(sys.argv) != 4:
	print "usage: %s suid fileIn fileOut" % sys.argv[0]
	sys.exit()

hashTail    = "2A00"
AppendixSize = 0x80+0x10;

suid    = sys.argv[1]
filein  = sys.argv[2]
fileout = sys.argv[3]
sourceType = filein[-3:]

assert filein != fileout, "filein and fileout cannot be the same"

with open(filein, "rb") as tmpfile:
	contents = tmpfile.read()
FileLen = len(contents)


# In the case of CFG -> CFC, add appendix
# 
if(sourceType.lower()=="cfg"):
	cfgSize = int(math.ceil(FileLen / 16.0) * 16)
	delta  = cfgSize - FileLen
	
	tail = list(binascii.unhexlify("43464300040080000000000004000000"))
	tail[8] = chr(FileLen%256)
	tail[9] = chr(FileLen/256)

	hash = hashlib.sha1(binascii.unhexlify(suid)[::-1])
	hash.update(binascii.unhexlify(hashTail))
	key = binascii.unhexlify(hash.hexdigest())[0:16]
	with open(fileout, "wb") as tmpfile:
		tmpfile.write(crypt(contents , key))
		tmpfile.write("\x00" * delta)		
		tmpfile.write("\x00" * 0x80)
		tmpfile.write("".join(tail))

else:
	tail = contents[FileLen-0x10:FileLen]
	cfgSize = int(binascii.hexlify(tail[11:7:-1]),16)
	ver1 = int(binascii.hexlify(tail[5:3:-1]),16)
	ver2 = int(binascii.hexlify(tail[13:11:-1]),16)
	signLen = int(binascii.hexlify(tail[7:5:-1]),16)

	hash = hashlib.sha1(binascii.unhexlify(suid)[::-1])
	hash.update(binascii.unhexlify(hashTail))
	key = binascii.unhexlify(hash.hexdigest())[0:16]
	with open(fileout, "wb") as tmpfile:
		tmpfile.write(crypt(contents[0:cfgSize] , key))



